(* ast.ml *)
(* An abstract syntax tree representing the GRAPL language *)
(* @authors Team GRAPL *)

type op = Add | Sub | Mult | Div | Equal | Neq | Less | Leq | Greater | Geq | Noop | Cons | Inc | Dec | Not
  
type expr = 
  | Literal of float
	| BooleanLiteral of string
  | Id of string 
  | Binop of expr*op*expr
	| Unop of op*expr
  | Assign of string*expr
  | Call of string*expr list
  | Unvisit of string
  | Visit of string
  | Noexpr 
  | List of expr list
  
type edge =
	| Redge of float
	| Ledge of float 
	| Bedge of float
	| NoEdge of float
		
type stmt =
  | Block of stmt list  	(* Statements *)
  | Expr of expr			(* { ... } *)
  | Foreach of string*string*string*string*(op*expr)*stmt (* foreach Unvisited/visited A from with B <10 {...}*)
  | While of expr*stmt
	| IfThenElse of expr*stmt*stmt (* if (foo == 42) then {} else {} *)
  | Graph of (string*edge*string) list (* graph [a >>4 b <>3 c << d];*)
	| Return of expr
  
type func_decl = {
  	rettype: string;
    fname : string;
    formals : (string*string) list;
    locals : (string*string) list;
    body : stmt list;
  }
  
type program = (string*string) list * func_decl list

	 
	